#include "scene.h"
#include <iostream.h>
#include <fstream.h>
#include <float.h>
#include <string.h>
#include "FL/gl.h"

#include "m33.h"

Scene *scene;

Scene::Scene() {

  tmeshN = 0;
  int w = 640;
  int h = 480;
  fb = new FrameBuffer(20, 40, w, h);
  fb->label("SW rendering");
  phc = new PHC(fb->w, fb->h, 60.0f);
  ui = new GraphAppUI();
  ui->GraphAppW->position(800, 40);

  animationStep = 0;

  // geometry
  tmeshN = 0;
  V3 newCenter;

  tmesh[0] = new TMesh('q');
  tmeshN = 1;

#if 0
  float maxDim;
  unsigned int color;
  maxDim = 75.0f;
  newCenter =  V3(50.0f, 0.0f, -200.0f);
  color = 0xFF00AAAA;
  tmesh[tmeshN] = new TMesh;
  tmesh[tmeshN]->Load("geo/chess.tris");
  tmesh[tmeshN]->ScaleToMaxDim(maxDim);
  tmesh[tmeshN]->TranslateTo(newCenter);
  tmesh[tmeshN]->SetColor(color);
  tmeshN++;
  
  maxDim = 75.0f;
  newCenter =  V3(0.0f, 0.0f, -100.0f);
  color = 0xFF0000AA;
  tmesh[tmeshN] = new TMesh;
  tmesh[tmeshN]->Load("geo/chess.tris");
  tmesh[tmeshN]->ScaleToMaxDim(maxDim);
  tmesh[tmeshN]->TranslateTo(newCenter);
  tmesh[tmeshN]->SetColor(color);
  tmeshN++;
#endif

  phc2 = 0;

  rstep = 3.0f;
  tstep = tmeshN ? tmesh[0]->FindDiagonal() / 10.0f : 1.0f;

  phc->Load("mydbg/view.txt");

  Show();
  Redraw();

}

void Scene::Show() {

  if (fb)
    fb->show();
  if (ui)
    ui->GraphAppW->show();

}

void Scene::UpdateScene() {

  V3 trans(1.0f, 0.0f, 0.0f);
  tmesh[1]->TranslateBy(trans);
  animationStep++;

}

void Scene::Redraw() {

  if (!ui->AnimationB->value()) {
    Render();
    fb->redraw();
    return;
  }

  while (ui->AnimationB->value()) {
    UpdateScene();
    Render();
    fb->redraw();
    Fl::check();
  }

}


void Scene::DBG() {

  phc->Save("mydbg/view.txt");
  return;

  V3 point = tmesh[0]->FindCenter(), vd(-1.0f, 0.0f, -1.0f), quasiUp(0.0f, 1.0f, 0.0f);
  float dist = 20.0f;
  vd = vd.Normalized();
  phc2 = new PHC(*phc);
  phc2->LookAt(point, vd, dist, quasiUp);
  Redraw();
  return;

  int neww = fb->w*8/4;
  int newh = fb->h*8/4;
  fb = new FrameBuffer(100, 100, neww, newh);
  fb->label("new resolution");
  fb->show();
  phc->ModifyResolution(neww);
  Redraw();
  return;

  M33 m;
  cerr << m;
  m[1][2] = 13.7f;
  cerr << m;
  return;

  V3 v(3.0f, 4.0f, 5.0f);
  V3 v2(1.0f, -1.0f, 3.0f);
  cerr << "v " << v << endl;
  cerr << "v2 " << v2 << endl;
  cerr << "v+v2 " << v+v2 << endl;
  cerr << "v-v2 " << v-v2 << endl;
  cerr << "v*v2 " << v*v2 << endl;
  cerr << "v length " << v.Length() << endl;
  cerr << "v normalized " << v.Normalized() << endl;

  return;

  if (fb) {
    fb->Clear(0xFFFFFFFF);
    float uf = 150.0f;
    float vf = 200.0f;
    int size = 53;
    fb->DrawPointSquare(uf, vf, size, 0xFF00FF00);
    fb->DrawPointDiagonalCross(uf, vf, size, 0xFF0000FF);
    float u1f = 170.4f;
    float v1f = 301.9f;
    fb->DrawSegment(uf, vf, u1f, v1f, 0xFFFF0000);
    unsigned int colors[2] = {0xFF00FF00, 0xFF0000FF};
    fb->MakeChecker(30, colors);
  }
  Redraw();

}

void Scene::Render() {

  if (!fb)
    return;

  unsigned int bgrColor = 0xFFFFFFFF;
  fb->Clear(bgrColor);

  if (phc2) {
    phc2->Render(fb, phc, 60.0f, 0xFFFF0000);
  }

  for (int tmi = 0; tmi < tmeshN; tmi++) {
    tmesh[tmi]->RenderFM(fb, phc);
  }


  return;


}

void Scene::MouseMove(int u, int v) {

  return;

  if (!fb)
    return;
  fb->DrawPointSquare(.5f+(float)u, .5f+(float)v, 5, 0xFF000000);
  Redraw();

}

void Scene::TranslateForward(float tstep) {

  phc->TranslateForward(tstep);
  Redraw();

}

void Scene::TranslateUp(float tstep) {

  phc->TranslateUp(tstep);
  Redraw();

}

void Scene::TranslateLeft(float tstep) {

  phc->TranslateLeft(tstep);
  Redraw();

}

void Scene::KeyPress(int key) {

    switch (key) {
    case '=': {
      tstep *= 2.0f;
      break;
    }
    case '-': {
      tstep /= 2.0f;
      break;
    }
    case '0': {
      rstep *= 2.0f;
      break;
    }
    case '9': {
      rstep /= 2.0f;
      break;
    }
    case 65362: {
      phc->TranslateForward(tstep);
      break;
    }
    case 65364: {
      phc->TranslateForward(-tstep);
      break;
    }
    case 65361: {
      phc->Pan(rstep);
      break;
    }
    case 65363: {
      phc->Pan(-rstep);
      break;
    }
    case 'w': {
      phc->Tilt(rstep);
      break;
    }
    case 's': {
      phc->Tilt(-rstep);
      break;
    }
    case 'm': {
      phc->Roll(rstep);
      break;
    }
    case 'n': {
      phc->Roll(-rstep);
      break;
    }
    case ',': {
      phc->TranslateLeft(tstep);
      break;
    }
    case '.': {
      phc->TranslateLeft(-tstep);
      break;
    }
    case 'q': {
      phc->TranslateUp(+tstep);
      break;
    }
    case 'a': {
      phc->TranslateUp(-tstep);
      break;
    }
    default:
      break;
  }

  phc->UpdateCamMatI();

  Redraw();

}